]> git.tuebingen.mpg.de Git - paraslash.git/blobdiff - server.c
gengetopt improvements
[paraslash.git] / server.c
index c46966ea8e2343f38366b14f34d360dce6d67026..8c9b48a6e371e27bad84baaa482556a4fda45550 100644 (file)
--- a/server.c
+++ b/server.c
@@ -21,8 +21,9 @@
 
 /** \mainpage Paraslash API Reference
  *
*  Good starting points for reading are probably \ref dbtool, \ref sender,
*  \ref receiver, \ref receiver_node, \ref filter, \ref filter_node.
  *
  */
 
@@ -47,29 +48,30 @@ INIT_SERVER_ERRLISTS;
 /** shut down non-authorized connections after that many seconds */
 #define ALARM_TIMEOUT 10
 
-/* these are exported to afs/command/dbtool */
+/* these are exported to afs.c. command.c and to all selectors */
 struct misc_meta_data *mmd;
 /** the configuration of para_server
  *
* It also contains the options for all database tools and all supported
  * senders.
 */
 struct gengetopt_args_info conf;
 char *user_list = NULL;
+extern void dccp_send_init(struct sender *);
 extern void http_send_init(struct sender *);
 extern void ortp_send_init(struct sender *);
 extern struct audio_format afl[];
 
-/** the list of supported database tools */
-struct dbtool dblist[] = {
+/** the list of supported audio file selectors */
+struct audio_file_selector dblist[] = {
        {
                .name = "random",
-               .init = random_dbtool_init,
+               .init = random_selector_init,
                .update_audio_file = NULL,
        },
        {
-               .name = "plm",
-               .init = plm_dbtool_init,
+               .name = "playlist",
+               .init = playlist_selector_init,
                .update_audio_file = NULL,
                .pre_select = NULL,
                .post_select = NULL,
@@ -77,7 +79,7 @@ struct dbtool dblist[] = {
 #ifdef HAVE_MYSQL
        {
                .name = "mysql",
-               .init = mysql_dbtool_init,
+               .init = mysql_selector_init,
                .update_audio_file = NULL,
                .pre_select = NULL,
                .post_select = NULL,
@@ -94,6 +96,10 @@ struct sender senders[] = {
                .name = "http",
                .init = http_send_init,
        },
+       {
+               .name = "dccp",
+               .init = dccp_send_init,
+       },
 #ifdef HAVE_ORTP
        {
                .name = "ortp",
@@ -117,7 +123,7 @@ static int signal_pipe;
  * \param ll the log level
  * \param fmt the format string describing the log message
  */
-void para_log(int ll, char* fmt,...)
+void para_log(int ll, const char* fmt,...)
 {
        va_list argp;
        FILE *outfd;
@@ -173,7 +179,7 @@ static void shm_init(void)
                goto err_out;
        mmd_mutex = ret;
 
-       mmd->dbt_num = 0;
+       mmd->selector_num = 0;
        mmd->num_played = 0;
        mmd->num_commands = 0;
        mmd->events = 0;
@@ -276,45 +282,29 @@ static void setup_signal_handling(void)
        }
 }
 
-static void init_dbtool(void)
+static void init_selector(void)
 {
-       int i;
+       int i, ret;
 
-       mmd->dbt_change = -1; /* no change nec., set to new dbt num by com_cdt */
-       if (!dblist[1].name)
+       mmd->selector_change = -1; /* no change nec., set to new num by com_chs */
+       if (!conf.selector_given)
                goto random;
-       if (conf.dbtool_given) {
-               for (i = 0; dblist[i].name; i++) {
-                       if (strcmp(dblist[i].name, conf.dbtool_arg))
-                               continue;
-                       PARA_NOTICE_LOG("initializing %s database tool\n",
-                               dblist[i].name);
-                       if (dblist[i].init(&dblist[i]) < 0) {
-                               PARA_WARNING_LOG("init %s failed",
-                                       dblist[i].name);
-                               goto random;
-                       }
-                       mmd->dbt_num = i;
-                       return;
-               }
-               PARA_WARNING_LOG("%s", "no such dbtool, switching to random\n");
-               goto random;
-       }
-       /* use the first dbtool that works
-        * (assuming that random always works)
-        */
-       for (i = 1; dblist[i].name; i++) {
-               int ret = dblist[i].init(&dblist[i]);
-               if (ret >= 0) {
-                       PARA_INFO_LOG("initialized %s\n", dblist[i].name);
-                       mmd->dbt_num = i;
-                       return;
+       for (i = 0; dblist[i].name; i++) {
+               if (strcmp(dblist[i].name, conf.selector_arg))
+                       continue;
+               PARA_NOTICE_LOG("initializing %s audio file selector\n",
+                       dblist[i].name);
+               ret = dblist[i].init(&dblist[i]);
+               if (ret < 0) {
+                       PARA_WARNING_LOG("%s", PARA_STRERROR(-ret));
+                       break;
                }
-               PARA_CRIT_LOG("%s init failed: %s\n", dblist[i].name,
-                       PARA_STRERROR(-ret));
+               mmd->selector_num = i;
+               return;
        }
+       PARA_WARNING_LOG("%s", "falling back to the random selector\n");
 random:
-       mmd->dbt_num = 0;
+       mmd->selector_num = 0;
        dblist[0].init(&dblist[0]); /* always successful */
 }
 
@@ -332,7 +322,7 @@ static void init_random_seed(void)
        int fd, ret = -1, len = sizeof(unsigned int);
        unsigned int seed;
 
-       fd = open("/dev/random", O_RDONLY);
+       fd = open("/dev/urandom", O_RDONLY);
        if (fd < 0)
                goto out;
        ret = -2;
@@ -367,7 +357,7 @@ static unsigned do_inits(int argc, char **argv)
        /* become daemon */
        if (conf.daemon_given)
                daemon_init();
-       init_dbtool();
+       init_selector();
        PARA_NOTICE_LOG("%s", "initializing audio file sender\n");
        /* audio file sender */
        afs_init();
@@ -385,21 +375,21 @@ static unsigned do_inits(int argc, char **argv)
        return sockfd;
 }
 
-static void handle_dbt_change(void)
+static void change_selector(void)
 {
-       int ret, old = mmd->dbt_num, new = mmd->dbt_change;
+       int ret, old = mmd->selector_num, new = mmd->selector_change;
 
        dblist[old].shutdown();
        ret = dblist[new].init(&dblist[new]);
-       mmd->dbt_change = -1; /* reset */
+       mmd->selector_change = -1; /* reset */
        if (ret >= 0) {
-               mmd->dbt_num = new;
+               mmd->selector_num = new;
                return;
        }
        /* init failed */
-       PARA_ERROR_LOG("%s -- switching to the random dbtool\n", PARA_STRERROR(-ret));
+       PARA_ERROR_LOG("%s -- switching to the random selector\n", PARA_STRERROR(-ret));
        dblist[0].init(&dblist[0]);
-       mmd->dbt_num = 0;
+       mmd->selector_num = 0;
 }
 
 /*
@@ -411,8 +401,8 @@ static void handle_sighup(void)
        close_log(logfile); /* gets reopened if necessary by parse_config */
        logfile = NULL;
        parse_config(1); /* reopens log */
-       mmd->dbt_change = mmd->dbt_num; /* do not change dbtool */
-       handle_dbt_change(); /* force reloading dbtool */
+       mmd->selector_change = mmd->selector_num; /* do not change selector.. */
+       change_selector(); /* .. just reload */
 }
 
 static void status_refresh(void)
@@ -474,8 +464,8 @@ repeat:
                        &max_fileno,
                        &rfds, &wfds);
        }
-       if (dblist[mmd->dbt_num].pre_select) {
-               ret = dblist[mmd->dbt_num].pre_select(&rfds, &wfds);
+       if (dblist[mmd->selector_num].pre_select) {
+               ret = dblist[mmd->selector_num].pre_select(&rfds, &wfds);
                max_fileno = MAX(max_fileno, ret);
        }
        mmd_unlock();
@@ -484,10 +474,10 @@ repeat:
        err = errno;
        //PARA_DEBUG_LOG("%s: select returned  %i\n", __func__, ret);
        mmd_lock();
-       if (mmd->dbt_change >= 0)
-               handle_dbt_change();
-       if (dblist[mmd->dbt_num].post_select)
-               dblist[mmd->dbt_num].post_select(&rfds, &wfds);
+       if (mmd->selector_change >= 0)
+               change_selector();
+       if (dblist[mmd->selector_num].post_select)
+               dblist[mmd->selector_num].post_select(&rfds, &wfds);
        if (ret < 0 && err == EINTR)
                goto repeat;
        if (ret < 0) {
@@ -522,7 +512,7 @@ repeat:
                case SIGTERM:
                        PARA_EMERG_LOG("terminating on signal %d\n", sig);
                        kill(0, SIGTERM);
-                       dblist[mmd->dbt_num].shutdown();
+                       dblist[mmd->selector_num].shutdown();
                        mutex_destroy(mmd_mutex);
                        shm_detach(mmd);
                        shm_destroy(mmd_shm_id);
@@ -21,8 +21,9 @@
 /** \mainpage Paraslash API Reference
  *
- *  Good starting points for reading are probably \ref dbtool, \ref sender,
- *  \ref receiver, \ref receiver_node, \ref filter, \ref filter_node.
+ *  Good starting points for reading are probably \ref audio_file_selector,
+ *  \ref sender, \ref receiver, \ref receiver_node, \ref filter, \ref
+ *  filter_node.
  *
  */
@@ -47,29 +48,30 @@ INIT_SERVER_ERRLISTS;
 /** the configuration of para_server
  *
- * It also contains the options for all database tools and all supported
+ * It also contains the options for all audio file selectors and all supported
  * senders.
 */